@@ -1,7 +1,4 @@
 # RDF::Query::Algebra::Filter
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -27,10 +24,9 @@ use RDF::Trine::Iterator qw(sgrep smap swatch);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -100,9 +96,7 @@ Returns the filter pattern.
 sub pattern {
 	my $self	= shift;
 	if (@_) {
-		my $pattern	= shift;
-		Carp::confess unless ($pattern->isa('RDF::Query::Algebra::GroupGraphPattern'));
-		$self->[2]	= $pattern;
+		$self->[2]	= shift;
 	}
 	return $self->[2];
 }
@@ -116,11 +110,13 @@ Returns the SSE string for this alegbra expression.
 sub sse {
 	my $self	= shift;
 	my $context	= shift;
+	my $prefix	= shift || '';
+	my $indent	= $context->{indent};
 	
 	return sprintf(
-		'(filter %s %s)',
-		$self->expr->sse( $context ),
-		$self->pattern->sse( $context ),
+		"(filter %s\n${prefix}${indent}%s)",
+		$self->expr->sse( $context, "${prefix}${indent}" ),
+		$self->pattern->sse( $context, "${prefix}${indent}" ),
 	);
 }
 
@@ -133,7 +129,7 @@ Returns the SPARQL string for this alegbra expression.
 sub as_sparql {
 	my $self	= shift;
 	my $context	= shift;
-	my $indent	= shift;
+	my $indent	= shift || '';
 	
 	my $expr	= $self->expr;
 	my $filter_sparql	= $expr->as_sparql( $context, $indent );
@@ -184,7 +180,7 @@ sub definite_variables {
 	return $pattern->definite_variables;
 }
 
-=item C<< fixup ( $bridge, $base, \%namespaces ) >>
+=item C<< fixup ( $query, $bridge, $base, \%namespaces ) >>
 
 Returns a new pattern that is ready for execution using the given bridge.
 This method replaces generic node objects with bridge-native objects.
@@ -194,47 +190,34 @@ This method replaces generic node objects with bridge-native objects.
 sub fixup {
 	my $self	= shift;
 	my $class	= ref($self);
+	my $query	= shift;
 	my $bridge	= shift;
 	my $base	= shift;
 	my $ns		= shift;
 	
-	my $expr	= $self->expr;
-	if ($expr->isa('RDF::Query::Algebra')) {
-		$expr	= $expr->fixup( $bridge, $base, $ns );
+	if (my $opt = $query->algebra_fixup( $self, $bridge, $base, $ns )) {
+		return $opt;
+	} else {
+		my $expr	= $self->expr;
+		if ($expr->isa('RDF::Query::Algebra')) {
+			$expr	= $expr->fixup( $query, $bridge, $base, $ns );
+		}
+		my $pattern	= $self->pattern->fixup( $query, $bridge, $base, $ns );
+		return $class->new( $expr, $pattern );
 	}
-	my $pattern	= $self->pattern->fixup( $bridge, $base, $ns );
-	return $class->new( $expr, $pattern );
 }
 
-=item C<< execute ( $query, $bridge, \%bound, $context, %args ) >>
+=item C<< is_solution_modifier >>
+
+Returns true if this node is a solution modifier.
 
 =cut
 
-sub execute {
-	my $self		= shift;
-	my $query		= shift;
-	my $bridge		= shift;
-	my $bound		= shift;
-	my $context		= shift;
-	my %args		= @_;
-	
-	my $expr		= $self->expr;
-	my $bool		= RDF::Query::Node::Resource->new( "sparql:ebv" );
-	my $filter		= RDF::Query::Expression::Function->new( $bool, $expr );
-	my $pattern		= $self->pattern;
-	my $stream		= sgrep {
-						my $bound	= $_;
-						my $bool	= 0;
-						eval {
-							my $value	= $filter->evaluate( $query, $bridge, $bound );
-							$bool	= ($value->literal_value eq 'true') ? 1 : 0;
-						};
-						return $bool;
-					} $pattern->execute( $query, $bridge, $bound, $context, %args );
+sub is_solution_modifier {
+	return 0;
 }
 
 
-
 1;
 
 __END__
