@@ -1,7 +1,4 @@
 # RDF::Query::Algebra::NamedGraph
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -16,9 +13,9 @@ use strict;
 use warnings;
 no warnings 'redefine';
 use base qw(RDF::Query::Algebra);
-use constant DEBUG	=> 0;
 
 use Data::Dumper;
+use Log::Log4perl;
 use RDF::Query::Error;
 use List::MoreUtils qw(uniq);
 use Carp qw(carp croak confess);
@@ -27,10 +24,9 @@ use RDF::Trine::Iterator qw(sgrep smap swatch);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -102,11 +98,13 @@ Returns the SSE string for this alegbra expression.
 sub sse {
 	my $self	= shift;
 	my $context	= shift;
+	my $prefix	= shift || '';
+	my $indent	= $context->{indent};
 	
 	return sprintf(
-		'(namedgraph %s %s)',
-		$self->graph->sse( $context ),
-		$self->pattern->sse( $context )
+		"(namedgraph\n${prefix}${indent}%s\n${prefix}${indent}%s)",
+		$self->graph->sse( $context, "${prefix}${indent}" ),
+		$self->pattern->sse( $context, "${prefix}${indent}" )
 	);
 }
 
@@ -198,7 +196,7 @@ sub qualify_uris {
 }
 
 
-=item C<< fixup ( $bridge, $base, \%namespaces ) >>
+=item C<< fixup ( $query, $bridge, $base, \%namespaces ) >>
 
 Returns a new pattern that is ready for execution using the given bridge.
 This method replaces generic node objects with bridge-native objects.
@@ -208,40 +206,21 @@ This method replaces generic node objects with bridge-native objects.
 sub fixup {
 	my $self	= shift;
 	my $class	= ref($self);
+	my $query	= shift;
 	my $bridge	= shift;
 	my $base	= shift;
 	my $ns		= shift;
 	
-	my $graph	= ($self->graph->isa('RDF::Query::Node'))
-				? $bridge->as_native( $self->graph )
-				: $self->graph->fixup( $bridge, $base, $ns );
-	
-	my $pattern	= $self->pattern->fixup( $bridge, $base, $ns );
-	return $class->new( $graph, $pattern );
-}
-
-=item C<< execute ( $query, $bridge, \%bound, $context, %args ) >>
-
-=cut
-
-sub execute {
-	my $self		= shift;
-	my $query		= shift;
-	my $bridge		= shift;
-	my $bound		= shift;
-	my $outer_ctx	= shift;
-	my %args		= @_;
-	
-	if ($outer_ctx) {
-		throw RDF::Query::Error::QueryPatternError ( -text => "Can't use nested named graphs" );
+	if (my $opt = $query->algebra_fixup( $self, $bridge, $base, $ns )) {
+		return $opt;
+	} else {
+		my $graph	= ($self->graph->isa('RDF::Query::Node'))
+					? $bridge->as_native( $self->graph )
+					: $self->graph->fixup( $query, $bridge, $base, $ns );
+		
+		my $pattern	= $self->pattern->fixup( $query, $bridge, $base, $ns );
+		return $class->new( $graph, $pattern );
 	}
-
-	my $context			= $self->graph;
-	my $named_triples	= $self->pattern;
-	
-	_debug( 'named triples: ' . Dumper($named_triples), 1 ) if (DEBUG);
-	my $nstream	= $named_triples->execute( $query, $bridge, $bound, $context, %args );
-	return $nstream;
 }
 
 1;
