@@ -1,7 +1,4 @@
 # RDF::Query::Algebra::Union
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -19,16 +16,16 @@ use base qw(RDF::Query::Algebra);
 
 use Data::Dumper;
 use Set::Scalar;
+use Log::Log4perl;
 use Scalar::Util qw(blessed);
 use List::MoreUtils qw(uniq);
 use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -107,11 +104,13 @@ Returns the SSE string for this alegbra expression.
 sub sse {
 	my $self	= shift;
 	my $context	= shift;
+	my $prefix	= shift || '';
+	my $indent	= $context->{indent};
 	
 	return sprintf(
-		'(union %s %s)',
-		$self->first->sse( $context ),
-		$self->second->sse( $context )
+		"(union\n${prefix}${indent}%s\n${prefix}${indent}%s)",
+		$self->first->sse( $context, "${prefix}${indent}" ),
+		$self->second->sse( $context, "${prefix}${indent}" )
 	);
 }
 
@@ -167,7 +166,7 @@ sub definite_variables {
 	return $seta->intersection( $setb )->members;
 }
 
-=item C<< fixup ( $bridge, $base, \%namespaces ) >>
+=item C<< fixup ( $query, $bridge, $base, \%namespaces ) >>
 
 Returns a new pattern that is ready for execution using the given bridge.
 This method replaces generic node objects with bridge-native objects.
@@ -177,52 +176,18 @@ This method replaces generic node objects with bridge-native objects.
 sub fixup {
 	my $self	= shift;
 	my $class	= ref($self);
+	my $query	= shift;
 	my $bridge	= shift;
 	my $base	= shift;
 	my $ns		= shift;
-	return $class->new( map { $_->fixup( $bridge, $base, $ns ) } $self->patterns );
-}
-
-=item C<< execute ( $query, $bridge, \%bound, $context, %args ) >>
-
-=cut
-
-sub execute {
-	my $self		= shift;
-	my $query		= shift;
-	my $bridge		= shift;
-	my $bound		= shift;
-	my $context		= shift;
-	my %args		= @_;
 	
-	my @names;
-	my @streams;
-	foreach my $u_triples ($self->first, $self->second) {
-		my $stream	= $u_triples->execute( $query, $bridge, $bound, $context, %args );
-		
-		if ($debug) {
-			$stream		= $stream->materialize;
-			warn "union stream:\n";
-			while (my $b = $stream->next) {
-				warn "BINDING: " . join(', ', map { my $v = $b->{$_}; join('=', $_, (blessed($v) ? $v->sse : '')) } (keys %$b));
-			}
-			$stream->reset;
-		}
-		
-		push(@names, $stream->binding_names);
-		push(@streams, $stream);
+	if (my $opt = $query->algebra_fixup( $self, $bridge, $base, $ns )) {
+		return $opt;
+	} else {
+		return $class->new( map { $_->fixup( $query, $bridge, $base, $ns ) } $self->patterns );
 	}
-	
-	@streams	= map { $_->project( @names ) } @streams;
-	my $stream	= shift(@streams);
-	while (@streams) {
-		$stream	= $stream->concat( shift(@streams), undef, \@names );
-	}
-	
-	return $stream;
 }
 
-
 1;
 
 __END__
