@@ -1,7 +1,4 @@
 # RDF::Query::Compiler::SQL
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -18,6 +15,7 @@ no warnings 'redefine';
 
 use RDF::Query::Error qw(:try);
 
+use Log::Log4perl;
 use List::Util qw(first);
 use Data::Dumper;
 use Math::BigInt;
@@ -31,10 +29,9 @@ use RDF::Query::Error qw(:try);
 ######################################################################
 
 my (@NODE_TYPE_TABLES, %NODE_TYPE_TABLES);
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 	@NODE_TYPE_TABLES	= (
 							['Resources', 'ljr', 'URI'],
 							['Literals', 'ljl', qw(Value Language Datatype)],
@@ -63,10 +60,11 @@ sub new {
 	my $class	= shift;
 	my $parsed	= shift;
 	my $model	= shift;
+	my $l		= Log::Log4perl->get_logger("rdf.query.compiler.sql");
 	my $stable;
 	if ($model) {
 		my $mhash	= _mysql_hash( $model );
-		warn "Model: $model => $mhash\n" if ($debug);
+		$l->debug("Model: $model => $mhash\n");
 		$stable		= "Statements${mhash}";
 	} else {
 		$stable		= 'Statements';
@@ -132,7 +130,7 @@ sub emit_select {
 	my $from	= $self->{from};
 	my $where	= $self->{where};
 	
-	my $options				= $parsed->{options} || {};
+	my $options				= $self->{options} || {};
 	my $unique				= $options->{'distinct'};
 	
 	my $from_clause;
@@ -190,8 +188,7 @@ sub order_by_clause {
 	
 	my $vars	= $self->{vars};
 	
-	my $parsed				= $self->{parsed};
-	my $options				= $parsed->{options} || {};
+	my $options				= $self->{options} || {};
 	my %variable_value_cols	= %$varcols;
 	
 	my $sql		= '';
@@ -260,6 +257,7 @@ contain the node values (for literals, resources, and blank nodes).
 
 sub add_variable_values_joins {
 	my $self	= shift;
+	my $l		= Log::Log4perl->get_logger("rdf.query.algebra.service");
 	my $parsed	= $self->{parsed};
 	my @vars	= map { $_->name } @{ $parsed->{variables} };
 	my %select_vars	= map { $_ => 1 } @vars;
@@ -281,7 +279,7 @@ sub add_variable_values_joins {
 		my $col_table	= (split(/[.]/, $col))[0];
 		my ($count)		= ($col_table =~ /\w(\d+)/);
 		
-		warn "var: $var\t\tcol: $col\t\tcount: $count\t\tunique count: $uniq_count\n" if ($debug);
+		$l->debug("var: $var\t\tcol: $col\t\tcount: $count\t\tunique count: $uniq_count\n");
 		
 		push(@cols, "${col} AS ${var}_Node") if ($select_vars{ $var });
 		foreach (@NODE_TYPE_TABLES) {
@@ -483,9 +481,27 @@ sub patterns2sql {
 		} elsif ($triple->isa('RDF::Query::Algebra::GroupGraphPattern')) {
 			++$$level;
 			$self->patterns2sql( [ $triple->patterns ], $level, %args );
+		} elsif ($triple->isa('RDF::Query::Algebra::Distinct')) {
+			$self->{options}{distinct}	= 1;
+			my $pattern	= $triple->pattern;
+			$self->patterns2sql( [ $pattern ], $level, %args );
+		} elsif ($triple->isa('RDF::Query::Algebra::Limit')) {
+			$self->{options}{limit}	= $triple->limit;
+			my $pattern	= $triple->pattern;
+			$self->patterns2sql( [ $pattern ], $level, %args );
+		} elsif ($triple->isa('RDF::Query::Algebra::Offset')) {
+			$self->{options}{offset}	= $triple->offset;
+			my $pattern	= $triple->pattern;
+			$self->patterns2sql( [ $pattern ], $level, %args );
+		} elsif ($triple->isa('RDF::Query::Algebra::Sort')) {
+			$self->{options}{orderby}	= [ $triple->orderby ];
+			my $pattern	= $triple->pattern;
+			$self->patterns2sql( [ $pattern ], $level, %args );
+		} elsif ($triple->isa('RDF::Query::Algebra::Project')) {
+			my $pattern	= $triple->pattern;
+			$self->patterns2sql( [ $pattern ], $level, %args );
 		} else {
-			my $op	= $triple->[0];
-			throw RDF::Query::Error::CompilationError( -text => "Unknown op '$op' in SQL compilation." );
+			throw RDF::Query::Error::CompilationError( -text => "Unknown pattern type '$triple' in SQL compilation." );
 		}
 	}
 	
