@@ -1,7 +1,4 @@
 # RDF::Query::Error
-# -------------
-# $Revision: 127 $
-# $Date: 2006-02-08 14:53:21 -0500 (Wed, 08 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -38,11 +35,9 @@ use base qw(Error);
 
 ######################################################################
 
-our ($REVISION, $VERSION, $debug);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$REVISION	= do { my $REV = (qw$Revision: 127 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
-	$VERSION	= 1.001;
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -99,6 +94,10 @@ package RDF::Query::Error::ExecutionError;
 
 use base qw(RDF::Query::Error);
 
+package RDF::Query::Error::RequestedInterruptError;
+
+use base qw(RDF::Query::Error);
+
 
 1;
 
