@@ -1,7 +1,4 @@
 # RDF::Query::Expression::Unary
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -24,10 +21,9 @@ use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -66,7 +62,7 @@ sub as_sparql {
 	my $context	= shift;
 	my $indent	= shift;
 	my $op		= $self->op;
-	return sprintf("$op %s", map { $_->as_sparql( $context, $indent ) } $self->operands);
+	return sprintf("($op %s)", map { $_->as_sparql( $context, $indent ) } $self->operands);
 }
 
 =item C<< evaluate ( $query, $bridge, \%bound ) >>
