@@ -1,7 +1,4 @@
 # RDF::Query::Functions
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -22,20 +19,19 @@ use RDF::Query;
 use RDF::Query::Model::RDFTrine;
 use RDF::Query::Error qw(:try);
 
+use Log::Log4perl;
 use I18N::LangTags;
-use Bloom::Filter;
 use Data::Dumper;
 use MIME::Base64;
-use Storable qw(thaw);
 use Digest::SHA1 qw(sha1_hex);
 use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION, $debug);
+our ($VERSION, $l);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$l			= Log::Log4perl->get_logger("rdf.query.functions");
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -268,8 +264,12 @@ $RDF::Query::functions{"http://www.w3.org/2001/XMLSchema#dateTime"}	= sub {
 $RDF::Query::functions{"sparql:str"}	= sub {
 	my $query	= shift;
 	my $bridge	= shift;
-	
 	my $node	= shift;
+	
+	unless (blessed($node)) {
+		throw RDF::Query::Error::TypeError -text => "STR() must be called with either a literal or resource";
+	}
+	
 	if ($node->is_literal) {
 		my $value	= $node->literal_value;
 		return RDF::Query::Node::Literal->new( $value );
@@ -288,30 +288,29 @@ $RDF::Query::functions{"sparql:logical-or"}	= sub {
 	### so that TypeErrors in arguments can be handled properly.
 	my $args	= shift;
 	
-	my $bool	= RDF::Query::Node::Resource->new( "sparql:ebv" );
-	my ($bool1, $bool2, $error);
-	try {
-		my $arg1 	= $args->();
-		my $func	= RDF::Query::Expression::Function->new( $bool, $arg1 );
-		my $value	= $func->evaluate( $query, $bridge, {} );
-		$bool1		= ($value->literal_value eq 'true') ? 1 : 0;
-	} otherwise {
-		warn "error in lhs of logical-or" if ($debug);
-		$error	= shift;
-	};
-	try {
-		my $arg2 	= $args->();
-		my $func	= RDF::Query::Expression::Function->new( $bool, $arg2 );
-		my $value	= $func->evaluate( $query, $bridge, {} );
-		$bool2		= ($value->literal_value eq 'true') ? 1 : 0;
-	} otherwise {
-		warn "error in rhs of logical-or" if ($debug);
-		$error	= shift;
-	};
+	my $ebv		= RDF::Query::Node::Resource->new( "sparql:ebv" );
+	my $arg;
+	my $error;
 	
-	if ($bool1 or $bool2) {
-		return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
-	} elsif ($error) {
+	while (1) {
+		my $bool;
+		try {
+			$arg 	= $args->();
+			if (defined($arg)) {
+				my $func	= RDF::Query::Expression::Function->new( $ebv, $arg );
+				my $value	= $func->evaluate( $query, $bridge, {} );
+				$bool		= ($value->literal_value eq 'true') ? 1 : 0;
+			}
+		} otherwise {
+			$l->debug("error in lhs of logical-or");
+			$error	||= shift;
+		};
+		last unless (defined($arg));
+		if ($bool) {
+			return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
+		}
+	}
+	if ($error) {
 		$error->throw;
 	} else {
 		return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
@@ -326,31 +325,32 @@ $RDF::Query::functions{"sparql:logical-and"}	= sub {
 	### so that TypeErrors in arguments can be handled properly.
 	my $args	= shift;
 	
-	my $bool	= RDF::Query::Node::Resource->new( "sparql:ebv" );
-	my ($bool1, $bool2, $error);
-	try {
-		my $arg1 = $args->();
-		my $func	= RDF::Query::Expression::Function->new( $bool, $arg1 );
-		my $value	= $func->evaluate( $query, $bridge, {} );
-		$bool1		= ($value->literal_value eq 'true') ? 1 : 0;
-	} otherwise {
-		$error	= shift;
-	};
-	try {
-		my $arg2 = $args->();
-		my $func	= RDF::Query::Expression::Function->new( $bool, $arg2 );
-		my $value	= $func->evaluate( $query, $bridge, {} );
-		$bool2		= ($value->literal_value eq 'true') ? 1 : 0;
-	} otherwise {
-		$error	= shift;
-	};
+	my $ebv		= RDF::Query::Node::Resource->new( "sparql:ebv" );
+	my $arg;
+	my $error;
 	
-	if ($bool1 and $bool2) {
-		return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
-	} elsif ($error) {
+	while (1) {
+		my $bool;
+		try {
+			$arg 	= $args->();
+			if (defined($arg)) {
+				my $func	= RDF::Query::Expression::Function->new( $ebv, $arg );
+				my $value	= $func->evaluate( $query, $bridge, {} );
+				$bool		= ($value->literal_value eq 'true') ? 1 : 0;
+			}
+		} otherwise {
+			$l->debug("error in lhs of logical-or");
+			$error	||= shift;
+		};
+		last unless (defined($arg));
+		unless ($bool) {
+			return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
+		}
+	}
+	if ($error) {
 		$error->throw;
 	} else {
-		return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
+		return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
 	}
 };
 
@@ -485,10 +485,10 @@ $RDF::Query::functions{"sparql:datatype"}	= sub {
 			throw RDF::Query::Error::TypeError ( -text => "cannot call datatype() on a language-tagged literal" );
 		} elsif ($node->has_datatype) {
 			my $type	= $node->literal_datatype;
-			warn "datatype => $type" if ($debug);
+			$l->debug("datatype => $type");
 			return RDF::Query::Node::Resource->new($type);
 		} else {
-			warn 'datatype => string' if ($debug);
+			$l->debug('datatype => string');
 			return RDF::Query::Node::Resource->new('http://www.w3.org/2001/XMLSchema#string');
 		}
 	} else {
@@ -648,13 +648,13 @@ $RDF::Query::functions{"java:com.hp.hpl.jena.query.function.library.listMember"}
 		if ($list->is_resource and $list->uri_value eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#nil') {
 			return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
 		} else {
-			my $stream	= $bridge->get_statements( $list, $first, undef );
+			my $stream	= $bridge->get_statements( $list, $first, undef, $query, {} );
 			while (my $stmt = $stream->next()) {
 				my $member	= $stmt->object;
 				return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean') if ($value->equal( $member ));
 			}
 			
-			my $stmt	= $bridge->get_statements( $list, $rest, undef )->next();
+			my $stmt	= $bridge->get_statements( $list, $rest, undef, $query, {} )->next();
 			return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean') unless ($stmt);
 			
 			my $tail	= $stmt->object;
@@ -726,13 +726,26 @@ $RDF::Query::functions{"http://kasei.us/2007/09/functions/warn"}	= sub {
 
 
 ### func:bloom( ?var, "frozen-bloom-filter" ) => true iff str(?var) is in the bloom filter.
+our $BLOOM_FILTER_LOADED;
+BEGIN {
+	$BLOOM_FILTER_LOADED	= do {
+		eval {
+			require Bloom::Filter;
+		};
+		($@)
+			? 0
+			: (Bloom::Filter->can('thaw'))
+				? 1
+				: 0;
+	};
+}
 {
 	my $BLOOM_URL	= 'http://kasei.us/code/rdf-query/functions/bloom';
 	sub _BLOOM_ADD_NODE_MAP_TO_STREAM {
 		my $query	= shift;
 		my $bridge	= shift;
 		my $stream	= shift;
-		warn "bloom filter got result stream\n" if ($debug);
+		$l->debug("bloom filter got result stream\n");
 		my $nodemap	= $query->{_query_cache}{ $BLOOM_URL }{ 'nodemap' };
 		$stream->add_extra_result_data('bnode-map', $nodemap);
 	}
@@ -744,7 +757,7 @@ $RDF::Query::functions{"http://kasei.us/2007/09/functions/warn"}	= sub {
 			$query->add_hook_once( 'http://kasei.us/code/rdf-query/hooks/post-execute', \&_BLOOM_ADD_NODE_MAP_TO_STREAM, "${BLOOM_URL}#add_node_map" );
 		}
 	} );
-	$RDF::Query::functions{"http://kasei.us/code/rdf-query/functions/bloom"}	= sub {
+	$RDF::Query::functions{ $BLOOM_URL }	= sub {
 		my $query	= shift;
 		my $bridge	= shift;
 		
@@ -752,6 +765,13 @@ $RDF::Query::functions{"http://kasei.us/2007/09/functions/warn"}	= sub {
 		my $filter	= shift;
 		my $bloom;
 		
+		unless ($BLOOM_FILTER_LOADED) {
+			$l->warn("Cannot compute bloom filter because Bloom::Filter is not available");
+			throw RDF::Query::Error::FilterEvaluationError ( -text => "Cannot compute bloom filter because Bloom::Filter is not available" );
+		}
+		
+		$l->debug("k:bloom being executed with node " . $value);
+		
 		if (exists( $query->{_query_cache}{ $BLOOM_URL }{ 'filters' }{ $filter } )) {
 			$bloom	= $query->{_query_cache}{ $BLOOM_URL }{ 'filters' }{ $filter };
 		} else {
@@ -760,11 +780,13 @@ $RDF::Query::functions{"http://kasei.us/2007/09/functions/warn"}	= sub {
 			$query->{_query_cache}{ $BLOOM_URL }{ 'filters' }{ $filter }	= $bloom;
 		}
 		
-		my @names	= RDF::Query::Algebra::Service->_names_for_node( $value, $query, $bridge, {}, 0 );
+		my $seen	= $query->{_query_cache}{ $BLOOM_URL }{ 'node_name_cache' }	= {};
+		my @names	= RDF::Query::Algebra::Service->_names_for_node( $value, $query, $bridge, {}, {}, 0, '', $seen );
+		$l->debug("- " . scalar(@names) . " identity names for node");
 		foreach my $string (@names) {
-			warn "checking bloom filter for --> '$string'\n" if ($debug);
+			$l->debug("checking bloom filter for --> '$string'\n");
 			my $ok	= $bloom->check( $string );
-			warn "-> ok\n" if ($ok and $debug);
+			$l->debug("-> ok") if ($ok);
 			if ($ok) {
 				my $nodemap	= $query->{_query_cache}{ $BLOOM_URL }{ 'nodemap' };
 				push( @{ $nodemap->{ $value->as_string } }, $string );
@@ -774,6 +796,42 @@ $RDF::Query::functions{"http://kasei.us/2007/09/functions/warn"}	= sub {
 		return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
 	};
 }
+
+$RDF::Query::functions{"http://kasei.us/code/rdf-query/functions/bloom/filter"}	= sub {
+	my $query	= shift;
+	my $bridge	= shift;
+	
+	my $value	= shift;
+	my $filter	= shift;
+	my $bloom;
+	
+	unless ($BLOOM_FILTER_LOADED) {
+		throw RDF::Query::Error::FilterEvaluationError ( -text => "Cannot compute bloom filter because Bloom::Filter is not available" );
+	}
+	
+	if (ref($query) and exists( $query->{_query_cache}{ "http://kasei.us/code/rdf-query/functions/bloom/filter" }{ 'filters' }{ $filter } )) {
+		$bloom	= $query->{_query_cache}{ "http://kasei.us/code/rdf-query/functions/bloom/filter" }{ 'filters' }{ $filter };
+	} else {
+		my $value	= $filter->literal_value;
+		$bloom	= Bloom::Filter->thaw( $value );
+		if (ref($query)) {
+			$query->{_query_cache}{ "http://kasei.us/code/rdf-query/functions/bloom/filter" }{ 'filters' }{ $filter }	= $bloom;
+		}
+	}
+	
+	my $string	= $value->as_string;
+	$l->debug("checking bloom filter for --> '$string'\n");
+	my $ok	= $bloom->check( $string );
+	$l->debug("-> ok") if ($ok);
+	if ($ok) {
+		return RDF::Query::Node::Literal->new('true', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
+	} else {
+		return RDF::Query::Node::Literal->new('false', undef, 'http://www.w3.org/2001/XMLSchema#boolean');
+	}
+};
+
+
+
 1;
 
 __END__
