@@ -1,13 +1,14 @@
 # RDF::Query::Node::Blank
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
 RDF::Query::Node::Blank - RDF Node class for blank nodes
 
+=head1 METHODS
+
+=over 4
+
 =cut
 
 package RDF::Query::Node::Blank;
@@ -23,10 +24,9 @@ use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
@@ -44,22 +44,28 @@ use overload	'<=>'	=> \&_cmp,
 sub _cmp {
 	my $nodea	= shift;
 	my $nodeb	= shift;
-	warn "blank comparison: " . Dumper($nodea, $nodeb) if ($debug);
+	my $l		= Log::Log4perl->get_logger("rdf.query.node.blank");
+	$l->debug("blank comparison: " . Dumper($nodea, $nodeb));
 	return 1 unless blessed($nodeb);
 	return -1 if ($nodeb->isa('RDF::Query::Node::Literal'));
 	return -1 if ($nodeb->isa('RDF::Query::Node::Resource'));
 	return 1 unless ($nodeb->isa('RDF::Query::Node::Blank'));
 	my $cmp	= $nodea->blank_identifier cmp $nodeb->blank_identifier;
-	warn "-> $cmp\n" if ($debug);
+	$l->debug("-> $cmp");
 	return $cmp;
 }
 
-=head1 METHODS
+=item C<< as_sparql >>
 
-=over 4
+Returns the SPARQL string for this node.
 
 =cut
 
+sub as_sparql {
+	my $self	= shift;
+	return $self->sse;
+}
+
 
 1;
 
