@@ -1,7 +1,4 @@
 # RDF::Query::Node::Resource
-# -------------
-# $Revision: 121 $
-# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -24,14 +21,20 @@ use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION);
 BEGIN {
-	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 }
 
 ######################################################################
 
+
+=head1 METHODS
+
+=over 4
+
+=cut
+
 use overload	'<=>'	=> \&_cmp,
 				'cmp'	=> \&_cmp,
 				'<'		=> sub { _cmp(@_) == -1 },
@@ -53,12 +56,18 @@ sub _cmp {
 	return $cmp;
 }
 
-=head1 METHODS
+=item C<< as_sparql >>
 
-=over 4
+Returns the SPARQL string for this node.
 
 =cut
 
+sub as_sparql {
+	my $self	= shift;
+	my $context	= shift;
+	return $self->sse( $context );
+}
+
 
 1;
 
