@@ -1,7 +1,4 @@
 # RDF::Query::Parser::RDQL
-# -------------
-# $Revision: 298 $
-# $Date: 2007-12-03 15:48:55 -0500 (Mon, 03 Dec 2007) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
@@ -25,12 +22,11 @@ use Scalar::Util qw(blessed);
 
 ######################################################################
 
-our ($VERSION, $debug, $lang, $languri);
+our ($VERSION, $lang, $languri);
 BEGIN {
 	$::RD_TRACE	= undef;
 	$::RD_HINT	= undef;
-	$debug		= 1;
-	$VERSION	= do { my $REV = (qw$Revision: 298 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '2.003_01';
 	$lang		= 'rdql';
 	$languri	= 'http://jena.hpl.hp.com/2003/07/query/RDQL';
 }
@@ -247,6 +243,16 @@ sub parse {
 	my $parsed	= $parser->query( $query );
 	
 	if ($parsed) {
+		my $pattern	= $parsed->{triples}[0];
+		if (blessed($pattern)) {
+			my $ns		= $parsed->{namespaces};
+			my $fixed	= $pattern->qualify_uris( $ns );
+			$parsed->{triples}[0]	= $fixed;
+		}
+		$pattern	= RDF::Query::Algebra::Project->new( $parsed->{triples}[0], $parsed->{variables} );
+		$parsed->{triples}[0]	= $pattern;
+		
+		
 		return $parsed;
 	} else {
 		return $self->fail( "Failed to parse: '$query'" );
